package eu.mhsl.minenet.minigames.instance.game.stateless.types.spleef;

import eu.mhsl.minenet.minigames.instance.game.stateless.StatelessGame;
import eu.mhsl.minenet.minigames.message.component.TranslatedComponent;
import eu.mhsl.minenet.minigames.score.LastWinsScore;
import eu.mhsl.minenet.minigames.util.BatchUtil;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.entity.GameMode;
import net.minestom.server.event.player.PlayerMoveEvent;
import net.minestom.server.event.player.PlayerStartDiggingEvent;
import net.minestom.server.instance.batch.AbsoluteBlockBatch;
import net.minestom.server.instance.block.Block;
import net.minestom.server.item.*;
import net.minestom.server.world.DimensionType;
import org.jetbrains.annotations.NotNull;

import java.util.concurrent.CompletableFuture;

public class Spleef extends StatelessGame {
    public Spleef() {
        super(DimensionType.OVERWORLD, "Spleef", new LastWinsScore());

        eventNode().addListener(PlayerStartDiggingEvent.class, this::destroyBlock);
    }

    @Override
    protected void onLoad(@NotNull CompletableFuture<Void> callback) {
        AbsoluteBlockBatch circle = new AbsoluteBlockBatch();

        for(int x = -20; x <= 20; x++) {
            for(int z = -20; z <= 20; z++) {
                circle.setBlock(x, 10, z, Block.SNOW_BLOCK);
            }
        }

        for(int x = -20; x <= 20; x++) {
            for(int z = -20; z <= 20; z++) {
                circle.setBlock(x, 20, z, Block.SNOW_BLOCK);
            }
        }

        for(int x = -20; x <= 20; x++) {
            for(int z = -20; z <= 20; z++) {
                circle.setBlock(x, 30, z, Block.SNOW_BLOCK);
            }
        }

        BatchUtil.loadAndApplyBatch(circle, this, () -> callback.complete(null));
    }

    @Override
    protected void onStart() {
        getPlayers().forEach(player -> {
            player.setGameMode(GameMode.SURVIVAL);
            player.getInventory().addItemStack(
                    ItemStack
                            .builder(Material.DIAMOND_SHOVEL)
                            .displayName(TranslatedComponent.byId("game_Spleef#shovelName").getAssembled(player))
                            .meta(
                                builder -> builder
                                    .enchantment(Enchantment.EFFICIENCY, (short) 99)
                                    .hideFlag(ItemHideFlag.HIDE_ENCHANTS)
                                    .build()
                            )
                            .build()
            );
            player.setHeldItemSlot((byte) 0);
        });
    }

    @Override
    protected void onPlayerMove(@NotNull PlayerMoveEvent playerMoveEvent) {
        if(playerMoveEvent.getNewPosition().y() < 9) {
            playerMoveEvent.getPlayer().setGameMode(GameMode.SPECTATOR);
            playerMoveEvent.getPlayer().getInventory().clear();
            getScore().addResult(playerMoveEvent.getPlayer());
        }
    }

    private void destroyBlock(PlayerStartDiggingEvent event) {
        setBlock(event.getBlockPosition(), Block.AIR);
    }

    @Override
    public Pos getSpawn() {
        return new Pos(0, 33, 0);
    }
}
