package eu.mhsl.minenet.minigames.instance.game.stateless.types.trafficlightrace;

import eu.mhsl.minenet.minigames.instance.game.stateless.StatelessGame;
import eu.mhsl.minenet.minigames.score.FirstWinsScore;
import eu.mhsl.minenet.minigames.util.BatchUtil;
import eu.mhsl.minenet.minigames.instance.Dimension;
import net.minestom.server.coordinate.Vec;
import net.minestom.server.event.player.PlayerMoveEvent;
import net.minestom.server.instance.batch.AbsoluteBlockBatch;
import net.minestom.server.instance.block.Block;
import net.minestom.server.timer.ExecutionType;
import net.minestom.server.timer.TaskSchedule;
import org.jetbrains.annotations.NotNull;

import java.util.concurrent.CompletableFuture;

class TrafficLightRace extends StatelessGame {
    private LightPhase phase = LightPhase.RED;
    private int phaseCounter = 1;

    public TrafficLightRace() {
        super(Dimension.THE_END.DIMENSION, "Ampelrennen", new FirstWinsScore());
    }

    @Override
    protected void onLoad(@NotNull CompletableFuture<Void> callback) {
        AbsoluteBlockBatch batch = new AbsoluteBlockBatch();
        for (int x = -10; x <= 10; x++) {
            for (int z = 5; z <= 100; z++) {
                batch.setBlock(x, 5, z, super.rnd.nextInt(0, 100) > 90 ? Block.SOUL_SAND : Block.BLACK_CONCRETE_POWDER);
            }
        }

        BatchUtil.loadAndApplyBatch(batch, this, () -> callback.complete(null));
    }

    @Override
    protected void onStart() {
        scheduler().submitTask(() -> {
            if(!super.isRunning) return TaskSchedule.stop();

            phaseCounter++;
            if(phaseCounter >= 4) phaseCounter = 0;

            if(phaseCounter == 0) phase = LightPhase.RED;
            if(phaseCounter == 1 || phaseCounter == 3) phase = LightPhase.YELLOW;
            if(phaseCounter == 2) phase = LightPhase.GREEN;

            getPlayers().forEach(player -> {
                for(int i = 0; i < 9; i++) {
                    player.getInventory().setItemStack(i, phase.item);
                }
            });

            return phase.taskScheduleRandomDuration();
        }, ExecutionType.SYNC);
    }

    @Override
    protected void onPlayerMove(@NotNull PlayerMoveEvent playerMoveEvent) {
        super.onPlayerMove(playerMoveEvent);

        if(playerMoveEvent.getNewPosition().z() > 110) stop();

        if(phase.equals(LightPhase.RED) && playerMoveEvent.getNewPosition().z() > playerMoveEvent.getPlayer().getPosition().z()) {
            playerMoveEvent.getPlayer().setVelocity(new Vec(0, 5, -10));
        }
    }

}
